/**
* Filename: BandyClientExampleMIDlet.java (in org.openbandy.example)
* This is part of the OpenBandy project.
* 
* www.openbandy.org
*/


package org.openbandy.example;

import java.io.IOException;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Image;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.openbandy.example.test.MessageSerializationTest;
import org.openbandy.example.test.PingPongTest;
import org.openbandy.example.test.RemoteStoreTest;
import org.openbandy.pref.Preferences;
import org.openbandy.pref.PreferencesForm;
import org.openbandy.service.LogService;
import org.openbandy.ui.MainMenu;
import org.openbandy.ui.StartupScreen;
import org.openbandy.ui.Tests;


/**
* An example of how to use the communication
* facilities in a MIDlet.
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
*/
public class BandyClientExampleMIDlet extends MIDlet {

	protected void startApp() throws MIDletStateChangeException {
		
		/* Get the display and create a startup screen that will be shown to the user during setup */
		Display display = Display.getDisplay(this);
		
		Image logo = null;
		try {
			logo = Image.createImage("/img/BandyStartupScreen.png");
		} catch (IOException ioe) { /* do nothing */ }
		
		StartupScreen startupScreen = new StartupScreen(logo, 6);
		display.setCurrent(startupScreen);
		
		
		/* ***** Mandatory part: Creating the main menu and the logger screen ***** */
		startupScreen.showActionAtStartupGauge("Creating main menu");
		
		/* create main menu and set it as the current screen */
		MainMenu mainMenu = new MainMenu(this, display);

		/* add the log to the main menu */
		startupScreen.showActionAtStartupGauge("Adding logger to main menu");
		mainMenu.addScreen(LogService.getLog());
		
		/* ***** Preferences: add two preferences as an example ***** */
		startupScreen.showActionAtStartupGauge("Adding two preferences");
		try {
			/* set the preferences */		
			Preferences.add("Username", true);
			Preferences.add("Phone Number", true);
			Preferences.add("Server Address", true);
		} catch (Exception e) {
			LogService.error(this, e.getMessage(), e);
		}
		
		/* add the preferences form to the main menu */
		startupScreen.showActionAtStartupGauge("Adding prefs to main menu");
		mainMenu.addScreen(new PreferencesForm());
		
		/* ***** Testing: add some test to the tests screen and add this to the main menu ***** */
		startupScreen.showActionAtStartupGauge("Creating tests");
		Tests tests = new Tests();
		tests.addTest(new MessageSerializationTest());
		tests.addTest(new PingPongTest());
		tests.addTest(new RemoteStoreTest());

		startupScreen.showActionAtStartupGauge("Adding tests to main menu");
		mainMenu.addScreen(tests);
		
		/* switch to main menu */
		display.setCurrent(mainMenu);
	}
	
	protected void destroyApp(boolean unconditional) throws MIDletStateChangeException {
		LogService.debug(this, "Bandy Example MIDlet destroyed");
	}

	protected void pauseApp() {
		LogService.debug(this, "Bandy Example MIDlet pausing..");
	}

}
