/**
 * Filename: Engine.java (in org.openbandy.example)
 * This file is part of the BandySerializer project.
 *
 * www.BandySerializer.org
 */

package org.openbandy.example;

import java.util.Vector;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializationBuffer;
import org.openbandy.io.SerializationIdentifiers;
import org.openbandy.io.Serializer;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class Engine implements Serializable {

	/* Storage serializer ID */
	private int engineId = SerializationIdentifiers.NO_ID;

	/* Attributes that will be serialized */
	private String layout = "";
	private int cubicCapacity = 0;
	private int cylinders = 0;

	/* References that will be serialized */
	private Crankshaft crankshaft;
	
	/* ************ Constructor ************ */

	public Engine(){
		/* ensure that no referenced object is null */
		if(crankshaft == null){
			crankshaft = new Crankshaft();
		}
	}

	
	/* ************ Getter and Setter Methods ************ */

	public String getLayout() {
		return layout;
	}

	public void setLayout(String layout) {
		this.layout = layout;
	}

	public int getCubicCapacity() {
		return cubicCapacity;
	}

	public void setCubicCapacity(int cubicCapacity) {
		this.cubicCapacity = cubicCapacity;
	}

	public int getCylinders() {
		return cylinders;
	}

	public void setCylinders(int cylinders) {
		this.cylinders = cylinders;
	}
	
	public Crankshaft getCrankshaft() {
		return crankshaft;
	}


	public void setCrankshaft(Crankshaft crankshaft) {
		this.crankshaft = crankshaft;
	}
	
	
	/* ************ Methods for interface Serializable ************ */

	public void serialize(Serializer serializer, SerializationBuffer serializationBuffer) {
		serializationBuffer.container().addString("layout", layout);
		serializationBuffer.container().addInt("cubicCapacity", cubicCapacity);
		serializationBuffer.container().addInt("cylinders", cylinders);
		
		serializer.serializeObject(crankshaft, serializationBuffer);
	}

	public void deserialize(Serializer serializer, SerializationBuffer serializationBuffer) {
		layout = serializationBuffer.container().getString("layout");
		cubicCapacity = serializationBuffer.container().getInt("cubicCapacity");
		cylinders = serializationBuffer.container().getInt("cylinders");
		
		serializer.deserializeObject(crankshaft, serializationBuffer);
	}

	public String getIdName() {
		return "engineId";
	}

	public int getIdValue() {
		return engineId;
	}

	public void setIdValue(int idValue) {
		engineId = idValue;
	}

	public void getAttributeNames(Vector attributes) {
		attributes.addElement("layout");
		attributes.addElement("cubicCapacity");
		attributes.addElement("cylinders");
	}

	public void getReferences(Vector references) {
		references.addElement(crankshaft);
	}

	public void getReferencedIdNames(Vector referencedIdNames) {
		referencedIdNames.addElement(crankshaft.getIdName());
	}

	public String getLegibleString() {
		return "Engine : "+layout+" CC:"+cubicCapacity+", Cylinders:"+cylinders+crankshaft.getLegibleString();
	}

	public boolean isEqualShallow(Serializable serializableObject) throws ClassCastException {
		Engine comparingEngine = (Engine)serializableObject;
		return ( (this.layout.equals(comparingEngine.layout)) &&
				(this.cubicCapacity == comparingEngine.cubicCapacity)&&
				(this.cylinders == comparingEngine.cylinders) );
	}

	public boolean isEqual(Serializable serializableObject) throws ClassCastException {
		if(isEqualShallow(serializableObject)){
			Engine comparingEngine = (Engine)serializableObject;
			return this.crankshaft.isEqual(comparingEngine.crankshaft);
		}
		return false;
	}

	public Serializable getCopy() {
		Engine clone = new Engine();
		clone.engineId = this.engineId;
		clone.layout = this.layout;
		clone.cubicCapacity = this.cubicCapacity;
		clone.cylinders = this.cylinders;
		clone.crankshaft = (Crankshaft)this.crankshaft.getCopy();
		return clone;
	}

	public Serializable getNewInstance() {
		return new Engine();
	}
}
