/**
* Filename: Tire.java (in org.openbandy.example)
* This is part of the OpenBandy project.
* 
* www.openbandy.org
*/

package org.openbandy.example;

import java.util.Vector;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializationBuffer;
import org.openbandy.io.SerializationIdentifiers;
import org.openbandy.io.Serializer;

/**
* An example business object that shows how to implement the Serializable
* interface. This class contains only one attribute.
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
*/
public class Tire implements Serializable{
    
    /* Storage serializer ID */
    private int tireId = SerializationIdentifiers.NO_ID;
    
    /* Attributes */
    private String dimension;
    private String mountedAt;
    
    /* ************ Constructors ************ */
    
    public Tire(String dimension, String mountedAt){
    	this.dimension = dimension;
    	this.mountedAt = mountedAt;
    }
    
    /* When an exlipcit constructor is defined,
     * you MUST also define the argumentless
     * constructor as serializers will always
     * invoke the constructor without arguments */
    public Tire(){
    	/* do nothing */
    }
    
    
    /* ************ Getter and Setter Methods ************ */

	public String getDimension() {
		return dimension;
	}

	public void setDimension(String dimension) {
		this.dimension = dimension;
	}

	public String getMountedAt() {
		return mountedAt;
	}

	public void setMountedAt(String mountedAt) {
		this.mountedAt = mountedAt;
	}

	
    /* ************ Methods for interface Serializable ************ */
	
	public void serialize(Serializer serializer, SerializationBuffer serializationBuffer) {
		serializationBuffer.container().addString("dimension", dimension);
		serializationBuffer.container().addString("mountedAt", mountedAt);
    }

    public void deserialize(Serializer serializer, SerializationBuffer serializationBuffer) {
    	dimension = serializationBuffer.container().getString("dimension");
    	mountedAt = serializationBuffer.container().getString("mountedAt");
    }

    public String getIdName() {
        return "tireId";
    }

    public int getIdValue() {
        return tireId;
    }

    public void setIdValue(int idValue) {
    	tireId = idValue;
    }

    public void getAttributeNames(Vector attributes) {
        attributes.addElement("dimension");
        attributes.addElement("mountedAt");
    }

    public void getReferences(Vector references) {
    	/* nothing to do */
    }

    public void getReferencedIdNames(Vector referenceNames) {
    	/* nothing to do */
    }

    public String getLegibleString() {
        return "Tire : "+dimension+", "+mountedAt;
    }

    public boolean isEqualShallow(Serializable serializableObject) throws ClassCastException {
        Tire comparingTire = (Tire)serializableObject;
        return (this.dimension.equals(comparingTire.dimension)
        		&& this.mountedAt.equals(comparingTire.mountedAt) );
    }
    
    public boolean isEqual(Serializable serializableObject) throws ClassCastException {
        return isEqualShallow(serializableObject);
    }

    public Serializable getCopy() {
        Tire clone = new Tire();
        clone.tireId = this.tireId;
        clone.dimension = this.dimension;
        clone.mountedAt = this.mountedAt;
        return clone;
    }
    
	public Serializable getNewInstance() {
		return new Tire();
	}
}
