/**
 * Filename: StringPreferenceTest.java (in org.openbandy.example.test)
 * This file is part of the BandyBasics project.
 *
 * www.BandyBasics.org
 */

package org.openbandy.example.test;

import java.util.Vector;

import org.openbandy.event.Event;
import org.openbandy.event.EventSink;
import org.openbandy.event.EventSource;
import org.openbandy.event.EventSourceOwner;
import org.openbandy.service.EventService;
import org.openbandy.service.LogService;
import org.openbandy.test.Test;

/**
 * A very simple test of the event router. This test assures that
 * only events of the type for which a sink has registered at the
 * event router are delegated. Moreover, in order to pass, this
 * test must check that the number of sent events corresponds with
 * the number of received events.
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class EventRouterTest extends Test implements EventSink, EventSourceOwner {

	private static final int NUMBER_OF_SIMULATED_EVENTS = 5;

	private int countReceivedEvents = 0;

	private EventSourceSimulation simulation;

	public EventRouterTest() {
		super("Event Router");
	}

	public void run() {
		testStarted();
		countReceivedEvents = 0;

		/* subscribe at the event router */
		EventService.subscribe(this, null);

		/* setup simulation */
		AllEventsSentMonitor monitor = new AllEventsSentMonitor();
		simulation = new EventSourceSimulation(monitor, this, NUMBER_OF_SIMULATED_EVENTS);

		/* start event simulation */
		simulation.start();

		/* wait until event simulation sent all events */
		LogService.debug(this, "EventSource simulation finished : "+monitor.areAllSent());

		if(countReceivedEvents == NUMBER_OF_SIMULATED_EVENTS) {
			testsScreen.setTestPassed(this);
		}
		else {
			LogService.warn(this, "Received incorrect number of events");
			testsScreen.setTestFailed(this);
		}

		testFinished();
	}

	/* *************** Methods for EventSink ***************** */

	public Vector getAbsorbedEventTypes() {
		Vector absorbedEventTypes = new Vector();
		absorbedEventTypes.addElement("TestEvent");
		return absorbedEventTypes;
	}

	public void absorb(Event event) {
		try {
			TestEvent testEvent = (TestEvent) event;
			countReceivedEvents++;
			LogService.debug(this, "Received event: "+testEvent.getEventMessage());
		} catch (ClassCastException cce) {
			LogService.error(this, "Test failed, received non TestEvent event", cce);
		}

	}

	public void lostEventSource(String eventShortClassName) {
		LogService.warn(this, "Test failed, lost event source for "+ eventShortClassName);
	}


	/* *************** Methods for EventSourceOwner ***************** */

	public void setNewInstance(EventSource newEventSourceInstance) {
		try{
			simulation = (EventSourceSimulation)newEventSourceInstance;
		}
		catch(ClassCastException cce){
			LogService.error(this, "New event source not an event simulation", cce);
		}
	}

}
