/**
 * Filename: Message.java (in org.openbandy.io)
 * This file is part of the BandyBasics project.
 *
 * www.openbandy.org
 */

//TODO this is the wrong place and the wrong concept for 'Message', but right now this is necessary
//in order to make the stream serialization of a message object with unknown content possible :-(

package org.openbandy.io;

import java.util.Vector;

import org.openbandy.io.stream.StreamSerializationBuffer;
import org.openbandy.io.stream.StreamSerializer;
import org.openbandy.log.LogImpl;
import org.openbandy.service.LogService;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public final class Message implements Serializable {

	/* Constants */
	private static final String NO_SENDER = "no sender";

	/* the name of the sender */
	private String sender;

	/* the type of the message */
	private int type;

	/* the messages content */
	private Serializable content;

	/**
	 * The default constructor set <code> NO_SENDER </code> and
	 * <code> UNDEFINED_TYPE </code>.
	 *  
	 */
	public Message(){
		this(NO_SENDER);
	}

	public Message(String sender){
		this.sender = sender;

		if(sender == null){
			LogService.warn(this, "Need sender");
		}
		/* Can not instantiate a new object for content, as
		 * we do not know of which concrete type it will be.
		 * Hence we have to do this in deserialize(). */
	}


	/**
	 * @return content The messages content.
	 */
	public Serializable getContent() {
		return content;
	}

	/**
	 * @return sender The originator if the message.
	 */
	public String getSender() {
		return sender;
	}

	/**
	 * @return type The arbitrary type of the message.
	 */
	public int getType() {
		return type;
	}

	/**
	 * TODO comment!
	 *
	 * @param type An arbitrary String describing the
	 *             type of the message.
	 */
	public void setType(int type) {
		this.type = type;
	}


	/**
	 * TODO comment!
	 *
	 * @param content
	 */
	public void setContent(Serializable content) {
		this.content = content;
	}


	/**
	 * TODO comment!
	 *
	 * @param sender
	 */
	public void setSender(String sender) {
		this.sender = sender;
		if(sender == null){
			LogService.warn(this, "Need sender");
		}
	}


	/* *************** Methods for Serialization ***************** */
	
	/* special method TODO comment */
	 public void prepare(Serializer serializer, SerializationBuffer buffer) {
	        if(content == null){
	            try{
	                StreamSerializer streamSerializer = (StreamSerializer) serializer;
	                StreamSerializationBuffer streamSerializationBuffer = (StreamSerializationBuffer)buffer;
	                content = streamSerializer.createContentObject(streamSerializationBuffer);
	            }catch(ClassCastException cce){
	                LogService.warn(this, "Need a stream serializer");
	            }
	        }
	        
	    }

	public void serialize(Serializer serializer, SerializationBuffer buffer) {
		buffer.container().addString("sender", sender);
		buffer.container().addInt("type", type);

		if(content != null){
			serializer.serializeObject(content, buffer);
		}
	}

	public void deserialize(Serializer serializer, SerializationBuffer buffer) {    
		sender = buffer.container().getString("sender");
		type = buffer.container().getInt("type");

		if(content != null){
			serializer.deserializeObject(content, buffer);
		}
	}

	public String getLegibleString() {
		String legibleString = sender+LogImpl.VALUE_DELIMITER+type;
		if(content != null){
			legibleString = legibleString+LogImpl.VALUE_DELIMITER+
			content.getLegibleString();
		}
		return legibleString;
	}

	public boolean isEqualShallow(Serializable serializableObject) throws ClassCastException {
		Message comparingMessage = (Message) serializableObject;
		return ( this.sender.equals(comparingMessage.getSender()) &&
				this.type == comparingMessage.getType() );
	}

	public boolean isEqual(Serializable serializableObject) throws ClassCastException {
		if(isEqualShallow(serializableObject)){
			Message comparingMessage = (Message) serializableObject;
			if( (content == null) && comparingMessage.content == null){
				return true;
			}
			if( (content != null) && (comparingMessage.content != null)){
				return comparingMessage.content.isEqual(content);
			}
			else{
				return false;
			}
		}
		return false;
	}

	public Serializable getNewInstance() {
		return new Message();
	}

	public Serializable getCopy() {
		Message clone = new Message();
		clone.sender = this.sender;
		clone.type = this.type;
		if(content != null){
			clone.content = (Serializable )content.getCopy();
		}
		return clone;
	}

	public void getAttributeNames(Vector attributes) {
		attributes.addElement("sender");
		attributes.addElement("type");
	}
	public void getReferences(Vector references) {
		if(content != null){
			references.addElement(content);
		}
	}
	public void getReferencedIdNames(Vector referenceNames) {
		if(content != null){
			referenceNames.addElement(content.getIdName());
		}
	}
	public String getIdName() {
		return null;
	}
	public int getIdValue() {
		return SerializationIdentifiers.NO_ID;
	}
	public void setIdValue(int idValue) {}


	/* ***** Overriding Methods ***** */

	public String toString() {
		return getLegibleString();
	}


}
